<?php
include_once 'models/Model.inc';

class Todo extends Model {
    private $description;
    private $done;

    public function __construct($id, $description, $done = false) {
        parent::__construct($id);
        $this -> setDescription($description);
        $this -> setDone($done);
    }

    public function getDescription() {
        return $this -> description;
    }

    public function setDescription($description) {
        $this -> description = $description;
        return $this;
    }

    public function getDone() {
        return $this -> done;
    }

    public function setDone($done) {
        $this -> done = $done;
        return $this;
    }

    static function findById($id) {
        $db = self::getDb();
        $st = $db -> prepare('SELECT * FROM todo WHERE id = :id');
        $st -> bindParam(':id', $id);
        $st -> execute();
        $row = $st -> fetch(PDO::FETCH_ASSOC);
        return self::makeTodoFromRow($row);
    }

    private static function makeTodoFromRow($row) {
        $todo = new Todo($row['id'], $row['description'], $row['done']);
        return $todo;
    }
    
    private static function makeTodosFromRows($rows) {
        $result = array();
        foreach ($rows as $row) {
            $result[] = self::makeTodoFromRow($row);
        }
        return $result;
    }

    static function findAllCurrent() {
        $db = self::getDb();
        $st = $db -> prepare('SELECT * FROM todo WHERE done = 0 ORDER BY id');
        $st -> execute();
        $rows = $st -> fetchAll(PDO::FETCH_ASSOC);
        return self::makeTodosFromRows($rows);
    }

    static function findAllDone() {
        $db = self::getDb();
        $st = $db -> prepare('SELECT * FROM todo WHERE done != 0 ORDER BY id');
        $st -> execute();
        $rows = $st -> fetchAll(PDO::FETCH_ASSOC);
        return self::makeTodosFromRows($rows);
    }

    function insert() {
        $db = self::getDb();
        $statement = $db -> prepare("INSERT INTO todo (description, done) values (:description, 0)");
        $statement -> bindParam(':description', $this -> description);
        $statement -> execute();
        $this -> id = $db -> lastInsertId();
        return $this;
    }

    function update() {
        $db = self::getDb();
        $statement = $db -> prepare("UPDATE todo SET description = :description, done = :done WHERE id = :id");
        $statement -> bindParam(':description', $this -> description);
        $statement -> bindParam(':done', $this -> done);
        $statement -> bindParam(':id', $this -> id);
        $statement -> execute();
        return $this;
    }

    function delete() {
        $db = self::getDb();
        $statement = $db -> prepare("DELETE FROM todo WHERE id = :id");
        $statement -> bindParam(':id', $this -> id);
        $statement -> execute();
    }

}
?>